pub mod api_keys;
pub mod auth;
mod signature;
pub mod users;

use api_keys::models::ApiKey as APIKey;
use axum::{Extension, Router};
use axum_jwt_login::AuthSessionWithApiKey;
use users::models::User;
use utoipa::OpenApi;
use utoipa_axum::router::OpenApiRouter;
use utoipa_redoc::{Redoc, Servable};
use utoipa_scalar::{Scalar, Servable as ScalarServable};
use utoipa_swagger_ui::SwaggerUi;

use super::{backend::ApiBackend, description::ApiDocumentation};

const API_BASE: &str = "/api";

type AuthBackendType = AuthSessionWithApiKey<User, ApiBackend, APIKey>;
#[macro_export]
macro_rules! login_required {
    () => {
        axum_jwt_login::login_required!(User, ApiBackend, APIKey)
    };
}
#[macro_export]
macro_rules! permission_required {
    ($($perm:expr),+ $(,)?) => {
        axum_jwt_login::permission_required!(
            crate::api::routes::User,
            crate::api::ApiBackend,
            crate::api::routes::APIKey,
            $($perm),+
        )
    };
}

pub fn create_routes(session: AuthBackendType) -> Router {
    let backend = session.backend();
    let (router, api) = OpenApiRouter::with_openapi(ApiDocumentation::openapi())
        // .routes(routes!(health))
        .nest(API_BASE, auth::router())
        .nest(API_BASE, users::router())
        .nest(API_BASE, api_keys::router())
        .nest(API_BASE, signature::router())
        //         .nest(
        //             "/api/order",
        //             // order::router().route_layer(crate::login_required!(AuthenticationBackend<ApiKey>)),
        //             order::router().route_layer(login_required!(User, auth_backend::AuthenticationBackend)),
        //         )
        //         .routes(routes!(
        //             inner::secret_handlers::get_secret,
        //             inner::secret_handlers::post_secret
        //         ))
        //         .nest("/api", users::router())
        //         // .layer(auth_layer)
        .layer(session.into_layer())
        .layer(Extension(backend))
        .split_for_parts();

    router
        .merge(SwaggerUi::new("/swagger-ui").url("/apidoc/openapi.json", api.clone()))
        .merge(Redoc::with_url("/redoc", api.clone()))
        .merge(Scalar::with_url("/scalar", api))
}
